@@ -320,7 +320,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	
 	stream = g_mime_stream_mem_new ();
 	gpg->diag = GMIME_STREAM_MEM (stream)->buffer;
-	charset = g_mime_charset_locale_name ();
+	charset = g_mime_locale_charset ();
 	if (strcasecmp (charset, "UTF-8") != 0) {
 		GMimeStream *fstream;
 		GMimeFilter *filter;
@@ -639,11 +639,16 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 		setsid ();
 		
 		maxfd = sysconf (_SC_OPEN_MAX);
-		/* Loop over all fds. */
-		for (i = 3; i < maxfd; i++) {
-			/* don't close status-fd or passwd-fd */
-			if (i != fds[7] && i != fds[8])
-				fcntl (i, F_SETFD, FD_CLOEXEC);
+		if (maxfd > 0) {
+			/* Loop over all fds. */
+			for (i = 0; i < maxfd; i++) {
+				if ((i != STDIN_FILENO) &&
+				    (i != STDOUT_FILENO) &&
+				    (i != STDERR_FILENO) &&
+				    (i != fds[7]) &&  /* status fd */
+				    (i != fds[8]))    /* passwd fd */
+					close (i);
+			}
 		}
 		
 		/* run gpg */
