@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 #include <config.h>
 #endif
 
+#include <string.h>
+
 #include "gmime-multipart-signed.h"
 #include "gmime-message-part.h"
 #include "gmime-stream-filter.h"
@@ -320,8 +322,7 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	g_mime_stream_reset (stream);
 	
 	/* construct the content part */
-	parser = g_mime_parser_new ();
-	g_mime_parser_init_with_stream (parser, stream);
+	parser = g_mime_parser_new_with_stream (stream);
 	content = g_mime_parser_construct_part (parser);
 	g_mime_stream_unref (stream);
 	g_object_unref (parser);
@@ -394,7 +395,7 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	
 	if (protocol) {
 		/* make sure the protocol matches the cipher sign protocol */
-		if (g_strcasecmp (ctx->sign_protocol, protocol) != 0)
+		if (strcasecmp (ctx->sign_protocol, protocol) != 0)
 			return NULL;
 	} else {
 		/* *shrug* - I guess just go on as if they match? */
@@ -405,7 +406,7 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	
 	/* make sure the protocol matches the signature content-type */
 	content_type = g_mime_content_type_to_string (signature->content_type);
-	if (g_strcasecmp (content_type, protocol) != 0) {
+	if (strcasecmp (content_type, protocol) != 0) {
 		g_mime_object_unref (signature);
 		g_free (content_type);
 		
