@@ -166,7 +166,7 @@ multipart_add_header (GMimeObject *object, const char *header, const char *value
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!g_strncasecmp ("Content-", header, 8))
+	if (!strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
 }
 
@@ -174,13 +174,13 @@ static void
 multipart_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	/* RFC 1864 states that you cannot set a Content-MD5 on a multipart */
-	if (!g_strcasecmp ("Content-MD5", header))
+	if (!strcasecmp ("Content-MD5", header))
 		return;
 	
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!g_strncasecmp ("Content-", header, 8))
+	if (!strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 }
 
@@ -190,7 +190,7 @@ multipart_get_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!g_strncasecmp ("Content-", header, 8))
+	if (!strncasecmp ("Content-", header, 8))
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
 	else
 		return NULL;
@@ -202,7 +202,7 @@ multipart_remove_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!g_strncasecmp ("Content-", header, 8))
+	if (!strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
@@ -762,7 +762,7 @@ g_mime_multipart_foreach (GMimeMultipart *multipart, GMimePartFunc callback, gpo
  * Returns the GMimeObject whose content-id matches the search string,
  * or %NULL if a match cannot be found.
  **/
-const GMimeObject *
+GMimeObject *
 g_mime_multipart_get_subpart_from_content_id (GMimeMultipart *multipart, const char *content_id)
 {
 	GMimeObject *object = (GMimeObject *) multipart;
@@ -771,20 +771,22 @@ g_mime_multipart_get_subpart_from_content_id (GMimeMultipart *multipart, const c
 	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), NULL);
 	g_return_val_if_fail (content_id != NULL, NULL);
 	
-	if (object->content_id && !strcmp (object->content_id, content_id))
+	if (object->content_id && !strcmp (object->content_id, content_id)) {
+		g_object_ref (object);
 		return object;
+	}
 	
 	subparts = multipart->subparts;
 	while (subparts) {
-		const GMimeObject *part = NULL;
+		GMimeObject *part = NULL;
 		GMimeObject *subpart;
 		
 		subpart = subparts->data;
 		
 		if (GMIME_IS_MULTIPART (subpart)) {
-			part = g_mime_multipart_get_subpart_from_content_id (GMIME_MULTIPART (subpart),
-									     content_id);
+			part = g_mime_multipart_get_subpart_from_content_id (GMIME_MULTIPART (subpart), content_id);
 		} else if (subpart->content_id && !strcmp (subpart->content_id, content_id)) {
+			g_object_ref (subpart);
 			part = subpart;
 		}
 		
