@@ -133,7 +133,7 @@ rfc2184_decode (const char *in, size_t len)
 	inptr++;
 	if (inptr < inend) {
 		len = inend - inptr;
-		if (g_strcasecmp (charset, "UTF-8") != 0) {
+		if (strcasecmp (charset, "UTF-8") != 0) {
 			char *udecoded;
 			iconv_t cd;
 			
@@ -144,7 +144,7 @@ rfc2184_decode (const char *in, size_t len)
 			if (cd == (iconv_t) -1) {
 				d(g_warning ("Cannot convert from %s to UTF-8, param display may "
 					     "be corrupt: %s", charset, g_strerror (errno)));
-				charset = g_mime_charset_locale_name ();
+				charset = g_mime_locale_charset ();
 				cd = g_mime_iconv_open ("UTF-8", charset);
 				if (cd == (iconv_t) -1)
 					return NULL;
@@ -395,7 +395,7 @@ decode_param (const char **in, char **paramp, char **valuep, gboolean *is_rfc218
 			 */
 			char *val;
 			
-			val = g_mime_utils_8bit_header_decode (value);
+			val = g_mime_utils_header_decode_text (value);
 			if (val) {
 				g_free (value);
 				value = val;
@@ -453,7 +453,7 @@ decode_param_list (const char **in)
 			break;
 		}
 		
-		if (is_rfc2184 && tail && !g_strcasecmp (name, tail->name)) {
+		if (is_rfc2184 && tail && !strcasecmp (name, tail->name)) {
 			/* rfc2184 allows a parameter to be broken into multiple parts
 			 * and it looks like we've found one. Append this value to the
 			 * last value.
@@ -644,7 +644,7 @@ encode_param (const unsigned char *in, gboolean *encoded)
 	if (!charset)
 		charset = "iso-8859-1";
 	
-	if (g_strcasecmp (charset, "UTF-8") != 0)
+	if (strcasecmp (charset, "UTF-8") != 0)
 		cd = g_mime_iconv_open (charset, "UTF-8");
 	
 	if (cd != (iconv_t) -1) {
