@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,12 +41,6 @@
 #define isblank(c) ((c) == ' ' || (c) == '\t')
 #endif /* HAVE_ISBLANK */
 
-#ifdef ENABLE_WARNINGS
-#define w(x) x
-#else
-#define w(x)
-#endif /* ENABLE_WARNINGS */
-
 #define d(x)
 
 static void g_mime_parser_class_init (GMimeParserClass *klass);
@@ -102,11 +96,12 @@ struct _GMimeParserPrivate {
 	off_t headers_start;
 	off_t header_start;
 	
-	unsigned int unstep:28;
+	unsigned int unstep:27;
 	unsigned int midline:1;
 	unsigned int seekable:1;
 	unsigned int scan_from:1;
 	unsigned int have_regex:1;
+	unsigned int persist_stream:1;
 	
 	GMimeContentType *content_type;
 	struct _header_raw *headers;
@@ -142,7 +137,7 @@ parser_push_boundary (GMimeParser *parser, const char *boundary)
 	} else {
 		s->boundary = g_strdup_printf ("--%s--", boundary);
 		s->boundarylen = strlen (boundary) + 2;
-		s->boundarylenfinal = strlen (s->boundary);
+		s->boundarylenfinal = s->boundarylen + 4;
 	}
 	
 	s->boundarylenmax = MAX (s->boundarylenfinal, max);
@@ -155,7 +150,7 @@ parser_pop_boundary (GMimeParser *parser)
 	struct _boundary_stack *s;
 	
 	if (!priv->bounds) {
-		w(g_warning ("boundary stack underflow"));
+		g_warning ("boundary stack underflow");
 		return;
 	}
 	
@@ -180,7 +175,7 @@ header_raw_find (struct _header_raw *headers, const char *name, off_t *offset)
 	
 	h = headers;
 	while (h) {
-		if (!g_strcasecmp (h->name, name)) {
+		if (!strcasecmp (h->name, name)) {
 			if (offset)
 				*offset = h->offset;
 			return h->value;
@@ -252,6 +247,7 @@ g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass)
 	
 	parser->priv->scan_from = FALSE;
 	parser->priv->have_regex = FALSE;
+	parser->priv->persist_stream = TRUE;
 }
 
 static void
@@ -408,6 +404,51 @@ g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
 }
 
 
+/**
+ * g_mime_parser_get_persist_stream:
+ * @parser: MIME parser object
+ *
+ * Gets whether or not the underlying stream is persistant.
+ *
+ * Returns %TRUE if the @parser will leave the content on disk or
+ * %FALSE if it will load the content into memory.
+ **/
+gboolean
+g_mime_parser_get_persist_stream (GMimeParser *parser)
+{
+	g_return_val_if_fail (GMIME_IS_PARSER (parser), FALSE);
+	
+	return (parser->priv->persist_stream && parser->priv->seekable);
+}
+
+
+/**
+ * g_mime_parser_set_persist_stream:
+ * @parser: MIME parser object
+ * @persist: persist attribute
+ *
+ * Sets whether or not the @parser's underlying stream is persistant.
+ *
+ * If @persist is %TRUE, the @parser will attempt to construct
+ * messages/parts whos content will remain on disk rather than being
+ * loaded into memory so as to reduce memory usage. This is the default.
+ *
+ * If @persist is %FALSE, the @parser will always load message content
+ * into memory.
+ *
+ * Note: This attribute only serves as a hint to the @parser. If the
+ * underlying stream does not support seeking, then this attribute
+ * will be ignored.
+ **/
+void
+g_mime_parser_set_persist_stream (GMimeParser *parser, gboolean persist)
+{
+	g_return_if_fail (GMIME_IS_PARSER (parser));
+	
+	parser->priv->persist_stream = persist;
+}
+
+
 /**
  * g_mime_parser_set_scan_from:
  * @parser: MIME parser object
@@ -675,7 +716,7 @@ parser_step_from (GMimeParser *parser)
 	header->name = g_strndup (priv->headerbuf, hlen);                 \
 	g_strstrip (header->name);                                        \
 	if (*colon != ':') {                                              \
-		w(g_warning ("Invalid header: %s", header->name));        \
+		g_warning ("Invalid header: %s", header->name);           \
 		header->value = header->name;                             \
 		header->name = g_strdup ("X-Invalid-Header");             \
 	} else {                                                          \
@@ -731,7 +772,7 @@ parser_step_headers (GMimeParser *parser)
 			while (*inptr != '\n')
 				inptr++;
 			
-			if (inptr == inend) {
+			if (inptr + 1 >= inend) {
 				/* we don't have enough data to tell if we
 				   got all of the header or not... */
 				priv->inptr = start;
@@ -769,6 +810,7 @@ parser_step_headers (GMimeParser *parser)
 	inend = priv->inend;
 	
 	header_backup (priv, inptr, inend - inptr);
+	/*header_parse (priv, hend);*/
 	
  headers_end:
 	
@@ -936,7 +978,7 @@ parser_scan_content (GMimeParser *parser, GByteArray *content)
 	
 	g_assert (priv->inptr <= priv->inend);
 	
-	inptr = priv->inptr;
+	start = inptr = priv->inptr;
 	
 	do {
 	refill:
@@ -1010,7 +1052,7 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	GMimeStream *stream;
 	off_t start, end;
 	
-	if (priv->seekable)
+	if (priv->persist_stream && priv->seekable)
 		start = parser_offset (parser, NULL);
 	else
 		content = g_byte_array_new ();
@@ -1018,17 +1060,17 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	*found = parser_scan_content (parser, content);
 	if (*found != FOUND_EOS) {
 		/* last '\n' belongs to the boundary */
-		if (priv->seekable)
+		if (priv->persist_stream && priv->seekable)
 			end = parser_offset (parser, NULL) - 1;
 		else
 			g_byte_array_set_size (content, MAX (content->len - 1, 0));
-	} else if (priv->seekable) {
+	} else if (priv->persist_stream && priv->seekable) {
 		end = parser_offset (parser, NULL);
 	}
 	
 	encoding = g_mime_part_get_encoding (mime_part);
 	
-	if (priv->seekable) {
+	if (priv->persist_stream && priv->seekable) {
 		stream = g_mime_stream_substream (priv->stream, start, end);
 	} else {
 		stream = g_mime_stream_mem_new_with_byte_array (content);
@@ -1230,13 +1272,12 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 		if (*found == FOUND_END_BOUNDARY && found_immediate_boundary (priv)) {
 			/* eat end boundary */
 			parser_skip_line (parser);
-			parser_pop_boundary (parser);
 			*found = parser_scan_multipart_postface (parser, multipart);
-		} else {
-			parser_pop_boundary (parser);
 		}
+		
+		parser_pop_boundary (parser);
 	} else {
-		w(g_warning ("multipart without boundary encountered"));
+		g_warning ("multipart without boundary encountered");
 		/* this will scan everything into the preface */
 		*found = parser_scan_multipart_preface (parser, multipart);
 	}
