@@ -28,20 +28,15 @@
 
 #define _GNU_SOURCE
 
-#include <glib.h>
-#ifdef G_OS_WIN32
-#include <process.h>
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h> /* for MAXHOSTNAMELEN */
 #else
 #define MAXHOSTNAMELEN 64
 #endif
 #include <sys/types.h>
-#include <string.h>
 #include <unistd.h>
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
@@ -55,6 +50,7 @@
 #include "gmime-charset.h"
 #include "gmime-iconv.h"
 #include "gmime-iconv-utils.h"
+#include "gmime-host-utils.h"
 
 #define d(x)
 #define w(x) x
@@ -334,7 +330,7 @@ get_wday (const unsigned char *in, unsigned int inlen)
 		return -1;
 	
 	for (wday = 0; wday < 7; wday++)
-		if (!g_strncasecmp (in, tm_days[wday], 3))
+		if (!strncasecmp (in, tm_days[wday], 3))
 			return wday;
 	
 	return -1;  /* unknown week day */
@@ -366,7 +362,7 @@ get_month (const unsigned char *in, unsigned int inlen)
 		return -1;
 	
 	for (i = 0; i < 12; i++)
-		if (!g_strncasecmp (in, tm_months[i], 3))
+		if (!strncasecmp (in, tm_months[i], 3))
 			return i;
 	
 	return -1;  /* unknown month */
@@ -764,7 +760,8 @@ g_mime_utils_generate_message_id (const char *fqdn)
 	static unsigned int count = 0;
 	char host[MAXHOSTNAMELEN + 1];
 #ifdef HAVE_GETHOSTBYNAME
-	struct hostent *h = NULL;
+	struct hostent hostbuf;
+	char *buf = NULL;
 #endif
 	char *msgid;
 	
@@ -772,7 +769,22 @@ g_mime_utils_generate_message_id (const char *fqdn)
 #ifdef HAVE_GETHOSTNAME
 		if (gethostname (host, sizeof (host)) == 0) {
 #ifdef HAVE_GETHOSTBYNAME
-			h = gethostbyname (host);
+			int ret, buflen = 1024, herr = 0;
+			
+			do {
+				if (buf != NULL)
+					buf = g_realloc (buf, buflen);
+				else
+					buf = g_malloc (buflen);
+				
+				if ((ret = g_gethostbyname_r (host, &hostbuf, buf, buflen, &herr)) == ERANGE)
+					buflen += 256;
+			} while (ret != 0 && buflen < 8192);
+			
+			if (ret != 0) {
+				g_free (buf);
+				buf = NULL;
+			}
 #endif /* HAVE_GETHOSTBYNAME */
 		} else {
 			host[0] = '\0';
@@ -782,7 +794,8 @@ g_mime_utils_generate_message_id (const char *fqdn)
 #endif /* HAVE_GETHOSTNAME */
 		
 #ifdef HAVE_GETHOSTBYNAME
-		fqdn = h ? h->h_name : (*host ? host : "localhost.localdomain");
+		fqdn = buf != NULL ? hostbuf.h_name : (*host ? host : "localhost.localdomain");
+		g_free (buf);
 #else
 		fqdn = *host ? host : "localhost.localdomain";
 #endif
@@ -1165,13 +1178,14 @@ g_mime_utils_unquote_string (char *string)
 gboolean
 g_mime_utils_text_is_8bit (const unsigned char *text, size_t len)
 {
-	const unsigned char *c, *inend;
+	register const unsigned char *inptr;
+	const unsigned char *inend;
 	
 	g_return_val_if_fail (text != NULL, FALSE);
 	
 	inend = text + len;
-	for (c = text; c < inend; c++)
-		if (*c > (unsigned char) 127)
+	for (inptr = text; *inptr && inptr < inend; inptr++)
+		if (*inptr > (unsigned char) 127)
 			return TRUE;
 	
 	return FALSE;
@@ -1310,14 +1324,14 @@ rfc2047_decode_word (const unsigned char *in, size_t inlen)
 			*p = '\0';
 		
 		/* slight optimization */
-		if (!g_strcasecmp (charset, "UTF-8"))
+		if (!strcasecmp (charset, "UTF-8"))
 			return g_strndup (decoded, declen);
 		
 		cd = g_mime_iconv_open ("UTF-8", charset);
 		if (cd == (iconv_t) -1) {
 			w(g_warning ("Cannot convert from %s to UTF-8, header display may "
 				     "be corrupt: %s", charset, g_strerror (errno)));
-			charset = g_mime_charset_locale_name ();
+			charset = g_mime_locale_charset ();
 			cd = g_mime_iconv_open ("UTF-8", charset);
 			if (cd == (iconv_t) -1)
 				return NULL;
@@ -1339,15 +1353,134 @@ rfc2047_decode_word (const unsigned char *in, size_t inlen)
 
 
 /**
- * g_mime_utils_8bit_header_decode:
+ * g_mime_utils_header_decode_text:
  * @in: header to decode
  *
- * Decodes and rfc2047 encoded header.
+ * Decodes an rfc2047 encoded 'text' header.
  *
- * Returns the mime encoded header as 8bit text.
+ * Returns the decoded header (which will be in UTF-8 if at all
+ * possible).
  **/
 char *
-g_mime_utils_8bit_header_decode (const unsigned char *in)
+g_mime_utils_header_decode_text (const unsigned char *in)
+{
+	GString *out, *lwsp, *text;
+	const unsigned char *inptr;
+	unsigned char *decoded;
+	gboolean last_was_encoded = FALSE;
+	gboolean last_was_space = FALSE;
+	
+	out = g_string_sized_new (256);
+	lwsp = g_string_sized_new (256);
+	text = g_string_sized_new (256);
+	inptr = in;
+	
+	while (inptr && *inptr) {
+		unsigned char c = *inptr++;
+		
+		if (is_lwsp ((int) c) && !last_was_space) {
+			/* we reached the end of an atom */
+			unsigned char *dword = NULL;
+			const unsigned char *word;
+			gboolean was_encoded;
+			
+			if ((was_encoded = is_rfc2047_encoded_word (text->str, text->len)))
+				word = dword = rfc2047_decode_word (text->str, text->len);
+			else
+				word = text->str;
+			
+			if (word) {
+				if (!(last_was_encoded && was_encoded)) {
+					/* rfc2047 states that you
+                                           must ignore all whitespace
+                                           between encoded words */
+					g_string_append (out, lwsp->str);
+				}
+				
+				g_string_append (out, word);
+				g_free (dword);
+			} else {
+				was_encoded = FALSE;
+				g_string_append (out, lwsp->str);
+				g_string_append (out, text->str);
+			}
+			
+			last_was_encoded = was_encoded;
+			
+			g_string_truncate (lwsp, 0);
+			g_string_truncate (text, 0);
+			
+			if (is_lwsp (c)) {
+				g_string_append_c (lwsp, c);
+				last_was_space = TRUE;
+			} else {
+				/* This is mostly here for interoperability with broken
+                                   mailers that might do something stupid like:
+                                   =?iso-8859-1?Q?blah?=:\t=?iso-8859-1?Q?I_am_broken?= */
+				g_string_append_c (out, c);
+				last_was_encoded = FALSE;
+				last_was_space = FALSE;
+			}
+			
+			continue;
+		}
+		
+		if (!is_lwsp (c)) {
+			g_string_append_c (text, c);
+			last_was_space = FALSE;
+		} else {
+			g_string_append_c (lwsp, c);
+			last_was_space = TRUE;
+		}
+	}
+	
+	if (text->len || lwsp->len) {
+		unsigned char *dword = NULL;
+		const unsigned char *word;
+		gboolean was_encoded;
+		
+		if ((was_encoded = is_rfc2047_encoded_word (text->str, text->len)))
+			word = dword = rfc2047_decode_word (text->str, text->len);
+		else
+			word = text->str;
+		
+		if (word) {
+			if (!(last_was_encoded && was_encoded)) {
+				/* rfc2047 states that you
+				   must ignore all whitespace
+				   between encoded words */
+				g_string_append (out, lwsp->str);
+			}
+			
+			g_string_append (out, word);
+			g_free (dword);
+		} else {
+			g_string_append (out, lwsp->str);
+			g_string_append (out, text->str);
+		}
+	}
+	
+	g_string_free (lwsp, TRUE);
+	g_string_free (text, TRUE);
+	
+	decoded = out->str;
+	g_string_free (out, FALSE);
+	
+	return (char *) decoded;
+}
+
+
+/**
+ * g_mime_utils_header_decode_phrase:
+ * @in: header to decode
+ *
+ * Decodes an rfc2047 encoded 'phrase' header.
+ *
+ * Returns the decoded header (which will be in UTF-8 if at all
+ * possible).
+ **/
+char *
+g_mime_utils_header_decode_phrase (const unsigned char *in)
 {
 	GString *out, *lwsp, *atom;
 	const unsigned char *inptr;
@@ -1495,8 +1628,7 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 	int save = 0;
 	char encoding;
 	
-	
-	if (g_strcasecmp (charset, "UTF-8") != 0)
+	if (strcasecmp (charset, "UTF-8") != 0)
 		cd = g_mime_iconv_open (charset, "UTF-8");
 	
 	if (cd != (iconv_t) -1) {
@@ -1514,7 +1646,7 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 	switch (g_mime_utils_best_encoding (word, len)) {
 	case GMIME_PART_ENCODING_BASE64:
 		enclen = BASE64_ENCODE_LEN (len);
-		encoded = g_alloca (enclen);
+		encoded = g_alloca (enclen + 1);
 		
 		encoding = 'b';
 		
@@ -1529,7 +1661,7 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 		break;
 	case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
 		enclen = QP_ENCODE_LEN (len);
-		encoded = g_alloca (enclen);
+		encoded = g_alloca (enclen + 1);
 		
 		encoding = 'q';
 		
@@ -1549,60 +1681,27 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 }
 
 
-/**
- * g_mime_utils_8bit_header_encode_phrase:
- * @in: header to encode
- *
- * Encodes a header phrase according to the rules in rfc2047.
- *
- * Returns the header phrase as 1 encoded atom. Useful for encoding
- * internet addresses.
- **/
-char *
-g_mime_utils_8bit_header_encode_phrase (const unsigned char *in)
-{
-	const char *charset;
-	GString *string;
-	size_t len;
-	char *str;
-	
-	if (in == NULL)
-		return NULL;
-	
-	len = strlen (in);
-	
-	charset = g_mime_charset_best (in, len);
-	charset = charset ? charset : "iso-8859-1";
-	
-	string = g_string_new ("");
-	
-	rfc2047_encode_word (string, in, strlen (in), charset, IS_ESAFE);
-	
-	str = string->str;
-	g_string_free (string, FALSE);
-	
-	return str;
-}
-
-
-enum _phrase_word_t {
+enum _rfc822_word_t {
 	WORD_ATOM,
+	WORD_QSTRING,
 	WORD_2047
 };
 
-struct _phrase_word {
-	struct _phrase_word *next;
+struct _rfc822_word {
+	struct _rfc822_word *next;
 	const unsigned char *start, *end;
-	enum _phrase_word_t type;
+	enum _rfc822_word_t type;
 	int encoding;
 };
 
 static gboolean
-word_types_compatable (enum _phrase_word_t type1, enum _phrase_word_t type2)
+word_types_compatable (enum _rfc822_word_t type1, enum _rfc822_word_t type2)
 {
 	switch (type1) {
 	case WORD_ATOM:
-		return FALSE;
+		return type2 != WORD_ATOM;
+	case WORD_QSTRING:
+		return type2 != WORD_2047;
 	case WORD_2047:
 		return type2 == WORD_2047;
 	default:
@@ -1610,16 +1709,17 @@ word_types_compatable (enum _phrase_word_t type1, enum _phrase_word_t type2)
 	}
 }
 
-static struct _phrase_word *
-rfc2047_encode_phrase_get_words (const unsigned char *in)
+/* okay, so 'text' fields don't actually contain 'word's, but we can group stuff similarly */
+static struct _rfc822_word *
+rfc2047_encode_get_rfc822_words (const unsigned char *in, gboolean phrase)
 {
 	const unsigned char *inptr, *start, *last;
-	struct _phrase_word *words, *tail, *word;
-	enum _phrase_word_t type = WORD_ATOM;
+	struct _rfc822_word *words, *tail, *word;
+	enum _rfc822_word_t type = WORD_ATOM;
 	int count = 0, encoding = 0;
 	
 	words = NULL;
-	tail = (struct _phrase_word *) &words;
+	tail = (struct _rfc822_word *) &words;
 	
 	last = start = inptr = in;
 	while (inptr && *inptr) {
@@ -1638,7 +1738,7 @@ rfc2047_encode_phrase_get_words (const unsigned char *in)
 		
 		if (g_unichar_isspace (c)) {
 			if (count > 0) {
-				word = g_new (struct _phrase_word, 1);
+				word = g_new (struct _rfc822_word, 1);
 				word->next = NULL;
 				word->start = start;
 				word->end = last;
@@ -1655,9 +1755,13 @@ rfc2047_encode_phrase_get_words (const unsigned char *in)
 			encoding = 0;
 		} else {
 			count++;
-			if (c > 127 && c < 256) {
+			if (phrase && c < 128) {
+				/* phrases can have qstring words */
+				if (!is_atom (c))
+					type = MAX (type, WORD_QSTRING);
+			} else if (c > 127 && c < 256) {
 				type = WORD_2047;
-				encoding = MAX (encoding, 2);
+				encoding = MAX (encoding, 1);
 			} else if (c >= 256) {
 				type = WORD_2047;
 				encoding = 2;
@@ -1668,7 +1772,7 @@ rfc2047_encode_phrase_get_words (const unsigned char *in)
 	}
 	
 	if (count > 0) {
-		word = g_new (struct _phrase_word, 1);
+		word = g_new (struct _rfc822_word, 1);
 		word->next = NULL;
 		word->start = start;
 		word->end = last;
@@ -1682,10 +1786,12 @@ rfc2047_encode_phrase_get_words (const unsigned char *in)
 	return words;
 }
 
+#define MERGED_WORD_LT_FOLDLEN(wlen, type) ((type) == WORD_2047 ? (wlen) < GMIME_FOLD_PREENCODED : (wlen) < (GMIME_FOLD_LEN - 8))
+
 static gboolean
-rfc2047_encode_phrase_merge_words (struct _phrase_word **wordsp)
+rfc2047_encode_merge_rfc822_words (struct _rfc822_word **wordsp)
 {
-	struct _phrase_word *word, *next, *words = *wordsp;
+	struct _rfc822_word *word, *next, *words = *wordsp;
 	gboolean merged = FALSE;
 	
 	/* scan the list, checking for words of similar types that can be merged */
@@ -1696,7 +1802,7 @@ rfc2047_encode_phrase_merge_words (struct _phrase_word **wordsp)
 		while (next) {
 			/* merge nodes of the same type AND we are not creating too long a string */
 			if (word_types_compatable (word->type, next->type)) {
-				if (next->end - word->start < GMIME_FOLD_PREENCODED) {
+				if (MERGED_WORD_LT_FOLDLEN (next->end - word->start, MAX (word->type, next->type))) {
 					/* the resulting word type is the MAX of the 2 types */
 					word->type = MAX (word->type, next->type);
 					
@@ -1727,21 +1833,42 @@ rfc2047_encode_phrase_merge_words (struct _phrase_word **wordsp)
 	return merged;
 }
 
-static char *
-rfc2047_encode_phrase (const unsigned char *in)
+static void
+g_string_append_len_quoted (GString *out, const char *in, size_t len)
 {
-	struct _phrase_word *words, *word, *prev = NULL;
+	register const char *inptr;
+	const char *inend;
+	
+	g_string_append_c (out, '"');
+	
+	inptr = in;
+	inend = in + len;
+	
+	while (inptr < inend) {
+		if ((*inptr == '"') || *inptr == '\\')
+			g_string_append_c (out, '\\');
+		
+		g_string_append_c (out, *inptr);
+		
+		inptr++;
+	}
+	
+	g_string_append_c (out, '"');
+}
+
+static char *
+rfc2047_encode (const unsigned char *in, gboolean phrase)
+{
+	struct _rfc822_word *words, *word, *prev = NULL;
+	gushort safemask = phrase ? IS_PSAFE : IS_ESAFE;
 	GString *out;
 	char *outstr;
 	
-	if (in == NULL)
-		return NULL;
-	
-	words = rfc2047_encode_phrase_get_words (in);
+	words = rfc2047_encode_get_rfc822_words (in, phrase);
 	if (!words)
 		return NULL;
 	
-	while (rfc2047_encode_phrase_merge_words (&words))
+	while (rfc2047_encode_merge_rfc822_words (&words))
 		;
 	
 	out = g_string_new ("");
@@ -1763,10 +1890,14 @@ rfc2047_encode_phrase (const unsigned char *in)
 		case WORD_ATOM:
 			g_string_append_len (out, word->start, word->end - word->start);
 			break;
+		case WORD_QSTRING:
+			g_assert (phrase);
+			g_string_append_len_quoted (out, word->start, word->end - word->start);
+			break;
 		case WORD_2047:
 			if (prev && prev->type == WORD_2047) {
 				/* include the whitespace chars between these 2 words in the
-                                   resulting rfc2047 encoded word. */
+				   resulting rfc2047 encoded word. */
 				len = word->end - prev->end;
 				start = prev->end;
 				
@@ -1778,10 +1909,9 @@ rfc2047_encode_phrase (const unsigned char *in)
 			}
 			
 			if (word->encoding == 1)
-				rfc2047_encode_word (out, start, len, "iso-8859-1", IS_PSAFE);
+				rfc2047_encode_word (out, start, len, "iso-8859-1", safemask);
 			else
-				rfc2047_encode_word (out, start, len,
-						     g_mime_charset_best (start, len), IS_PSAFE);
+				rfc2047_encode_word (out, start, len, g_mime_charset_best (start, len), safemask);
 			break;
 		}
 		
@@ -1799,19 +1929,98 @@ rfc2047_encode_phrase (const unsigned char *in)
 }
 
 
+/**
+ * g_mime_utils_header_encode_phrase:
+ * @in: header to encode
+ *
+ * Encodes a 'phrase' header according to the rules in rfc2047.
+ *
+ * Returns the encoded 'phrase'. Useful for encoding internet
+ * addresses.
+ **/
+char *
+g_mime_utils_header_encode_phrase (const unsigned char *in)
+{
+	if (in == NULL)
+		return NULL;
+	
+	return rfc2047_encode (in, TRUE);
+}
+
+
+/**
+ * g_mime_utils_header_encode_text:
+ * @in: header to encode
+ *
+ * Encodes a 'text' header according to the rules in rfc2047.
+ *
+ * Returns the encoded header. Useful for encoding
+ * headers like "Subject".
+ **/
+char *
+g_mime_utils_header_encode_text (const unsigned char *in)
+{
+	if (in == NULL)
+		return NULL;
+	
+	return rfc2047_encode (in, FALSE);
+}
+
+
+/**
+ * g_mime_utils_8bit_header_decode:
+ * @in: header to decode
+ *
+ * Decodes an rfc2047 encoded header.
+ *
+ * Note: This function is deprecated. Use
+ * #g_mime_utils_header_decode_text instead.
+ *
+ * Returns the decoded header (which will be in UTF-8 if at all
+ * possible).
+ **/
+char *
+g_mime_utils_8bit_header_decode (const unsigned char *in)
+{
+	return g_mime_utils_header_decode_text (in);
+}
+
+
 /**
  * g_mime_utils_8bit_header_encode:
  * @in: header to encode
  *
- * Encodes a header according to the rules in rfc2047.
+ * Encodes a 'text' header according to the rules in rfc2047.
  *
- * Returns the header as several encoded atoms. Useful for encoding
+ * Note: This function is deprecated. Use
+ * #g_mime_utils_header_encode_text instead.
+ *
+ * Returns the encoded header. Useful for encoding
  * headers like "Subject".
  **/
 char *
 g_mime_utils_8bit_header_encode (const unsigned char *in)
 {
-	return rfc2047_encode_phrase (in);
+	return g_mime_utils_header_encode_text (in);
+}
+
+
+/**
+ * g_mime_utils_8bit_header_encode_phrase:
+ * @in: header to encode
+ *
+ * Encodes a 'phrase' header according to the rules in rfc2047.
+ *
+ * Note: This function is deprecated. Use
+ * #g_mime_utils_header_encode_phrase instead.
+ *
+ * Returns the encoded 'phrase'. Useful for encoding internet
+ * addresses.
+ **/
+char *
+g_mime_utils_8bit_header_encode_phrase (const unsigned char *in)
+{
+	return g_mime_utils_header_encode_phrase (in);
 }
 
 
